'use strict';
module.exports.create = function (config) {
    var baseUrl = config.baseUrl.replace(/\/$/, '');
    var authtoken = config.token;
    var request;
    return {
        init: function (opts) {
            request = opts.request;
            return null;
        },
        zones: function (opts) {
            return request({
                method: 'GET',
                url: baseUrl + '/zones',
                headers: {
                    'X-Api-Key': authtoken
                },
                json: true
            }).then(function (resp) {
                return resp.body.map(function (zone) {
                    return zone.name;
                });
            });
        },
        set: function (opts) {
            return request({
                method: 'GET',
                url: baseUrl +
                    '/zones/' +
                    opts.challenge.altname.match(/[A-Za-z0-9]*\.[A-Za-z0-9]*$/)[0],
                headers: {
                    'X-Api-Key': authtoken
                },
                json: true
            }).then(function (resp) {
                let a = resp.body.rrsets;
                let inserted = false;
                let records = [];
                for (let i = 0; i < a.length; i++) {
                    if (a[i].name === opts.challenge.dnsHost + '.') {
                        records = a[i].records;
                        inserted = true;
                        break;
                    }

                }
                if (inserted) {
                    for (let i = 0; i < records.length; i++) {
                        records[i].type = "TXT";
                        records[i].ttl = 300;
                        records[i].name = opts.challenge.dnsHost;
                    }
                    records.push({
                        "content": '"' + opts.challenge.dnsAuthorization + '"',
                        "disabled": false,
                        "ttl": 300,
                        "name": opts.challenge.dnsHost + ".",
                        "type": "TXT"
                    })
                } else {
                    records.push({
                        "content": '"' + opts.challenge.dnsAuthorization + '"',
                        "disabled": false,
                        "ttl": 300,
                        "name": opts.challenge.dnsHost + ".",
                        "type": "TXT"
                    })
                }
                return request({
                    method: 'PATCH',
                    url: baseUrl +
                        '/zones/' +
                        opts.challenge.hostname.match(/[A-Za-z0-9]*\.[A-Za-z0-9]*$/)[0],
                    body: {
                        "rrsets": [{
                            "name": opts.challenge.dnsHost + ".",
                            "type": "TXT",
                            "ttl": 300,
                            "changetype": "REPLACE",
                            "records": records
                        }]
                    },
                    headers: {
                        'X-Api-Key': authtoken
                    },
                    json: true
                }).then(function (resp) {
                    return true;
                });
            });
        },
        remove: function (opts) {
            return request({
                method: 'PATCH',
                url: baseUrl +
                    '/zones/' +
                    opts.challenge.hostname.match(/[A-Za-z0-9]*\.[A-Za-z0-9]*$/)[0],
                body: {
                    "rrsets": [{
                        "name": opts.challenge.dnsHost + ".",
                        "type": "TXT",
                        "changetype": "DELETE",
                        "records": []
                    }]
                },
                headers: {
                    'X-Api-Key': authtoken
                },
                json: true
            }).then(function (resp) {
                return resp;
            });
        },
        get: function (opts) {
            return request({
                method: 'GET',
                url: baseUrl +
                    '/zones/' +
                    opts.challenge.altname.match(/[A-Za-z0-9]*\.[A-Za-z0-9]*$/)[0],
                headers: {
                    'X-Api-Key': authtoken
                },
                json: true
            }).then(function (resp) {
                let a = resp.body.rrsets;
                for (let i = 0; i < a.length; i++) {
                    if (a[i].name === opts.challenge.identifier.value + '.') {
                        for (let j = 0; j < a[i].records.length; j++) {
                            let r = a[i].records[j].content.substr(1, a[i].records[j].content.length - 2)
                            if (r === opts.challenge.dnsAuthorization) {
                                let c = {
                                    "dnsAuthorization": r
                                }
                                return c;
                            }
                        }
                    }
                }
                return null;
            });
        }
    };
};